package com.vendor.burn.serial.workers;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.analysis.TimeLogger;
import com.vendor.burn.commands.EraseCommand;
import com.vendor.burn.commands.I2cCommand;
import com.vendor.burn.commands.MWCommand;
import com.vendor.burn.commands.SFCommand;
import com.vendor.burn.commands.SerialCommand;
import com.vendor.burn.commands.UnZipCommand;
import com.vendor.burn.commands.WriteCommand;
import com.vendor.burn.model.ChipID;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.FlashType;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.PlatformType;
import com.vendor.burn.model.TransferWay;
import com.vendor.burn.model.config.BurnConfig;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.devices.IDeviceSetting;
import com.vendor.burn.model.devices.burners.AbstractBurner;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.model.event.EventType;
import com.vendor.burn.model.receive.ReceiveTimeOutException;
import com.vendor.burn.model.struct.XmodelParameter;
import com.vendor.burn.preferences.FastbootPreferences;
import com.vendor.burn.serial.workers.AddressBurnerWorker;
import com.vendor.burn.serial.workers.CallLibJTAGWorker;
import com.vendor.burn.serial.workers.CallLibUSBWorker;
import com.vendor.burn.serial.workers.WorkerUtility;
import com.vendor.burn.sparse.SparseFileAgent;
import com.vendor.burn.thirdparty.ZlibCRC32;
import com.vendor.burn.utilitily.BufferedRandomAccessFile;
import com.vendor.burn.utilitily.BytesUtils;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.GZipUtils;
import com.vendor.burn.utilitily.StringUtils;
import com.vendor.burn.utilitily.ToolUtils;
import com.vendor.burn.views.FastbootView;
import com.vendor.platform.api.statistics.ToolStatistics;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.osgi.util.NLS;

public class PartitionBurnerWorker extends AddressBurnerWorker {
  private static long bootargsFileLen = 0L;
  
  protected static final int SECTOR_SIZE = 512;
  
  protected static final int UFS_SIZE = 4096;
  
  private static long tempFileDivisionSize = 0L;
  
  protected String tmpBootargsString;
  
  protected static final int UIMONITOR_100 = 100;
  
  protected static final int UIMONITOR_50 = 50;
  
  protected static final int UIMONITOR_1 = 1;
  
  protected static final int EMPTY_LEN = 5242880;
  
  public PartitionBurnerWorker(IBurner burner) {
    super(burner);
  }
  
  public boolean burn(IProgressMonitor uimonitor, List<PartitionItem> parts) throws IOException, ReceiveTimeOutException {
    ToolStatistics.INSTANCE.logUsage("BurnTool", "Burn Partition", "ChipName: " + this.burner
        .getBurnContext().getDeviceSetting().getName());
    ToolStatistics.INSTANCE.logToolInUse("BurnTool");
    String versionInfo = getVersionInfo("Burn Partition");
    CallLibJTAGWorker.printDebugInfo(versionInfo);
    if (UnknownDevice.isOpenLightDebug())
      initLightState(); 
    this.tftpTimeLogger = new TimeLogger();
    this.burnTimeLogger = new TimeLogger();
    long burnStartTime = 0L;
    burnStartTime = System.currentTimeMillis();
    setProgressMonitor(uimonitor);
    getMonitor().getEvent().init(EventType.BURN);
    int selectedPartCount = 0;
    for (PartitionItem partitionItem : parts) {
      if (partitionItem.isChecked())
        selectedPartCount++; 
    } 
    uimonitor.beginTask("Burn By Partition", 100 * selectedPartCount);
    try {
      ArrayList<FlashType> hasFlashTypes = new ArrayList<>();
      for (PartitionItem partitionItem : parts) {
        if (partitionItem.isChecked()) {
          FlashType flashType = partitionItem.getFlashType();
          if (!hasFlashTypes.contains(flashType))
            hasFlashTypes.add(flashType); 
        } 
      } 
      boolean result = burnByTransferWay(uimonitor, parts, selectedPartCount, hasFlashTypes);
      printLog(burnStartTime);
      return result;
    } finally {
      printPartsCrcValue(parts);
      uimonitor.done();
    } 
  }
  
  private void printPartsCrcValue(List<PartitionItem> parts) {
    if (parts == null)
      return; 
    for (PartitionItem partitionItem : parts) {
      if (partitionItem.isChecked()) {
        File file = partitionItem.getFile();
        if (file != null)
          printCrcValue(file); 
      } 
    } 
  }
  
  private boolean burnByTransferWay(IProgressMonitor uimonitor, List<PartitionItem> parts, int selectedPartCount, ArrayList<FlashType> hasFlashTypes) throws IOException, ReceiveTimeOutException {
    PartitionItem fastboot = parts.get(0);
    boolean result = true;
    TransferWay curTransferWay = getBurner().getBurnContext().getTransferWay();
    if (curTransferWay == TransferWay.Uart2Compress) {
      result = burnByUart2Compress(uimonitor, parts, hasFlashTypes, fastboot, curTransferWay);
      if (!result)
        return result; 
      sendCommandToSetMac();
      String resetCommand = "reset";
      sendCommandNoReturnValue(resetCommand);
    } else if (curTransferWay == TransferWay.Usb2I2c || curTransferWay == TransferWay.I2CBootrom) {
      this.i2cTimeLogger = new TimeLogger();
      this.flashTimeLogger = new TimeLogger();
      this.resetTimeLogger = new TimeLogger();
      result = burnByI2c(uimonitor, parts, fastboot, result, hasFlashTypes, selectedPartCount);
      if (ToolUtils.isDebugOn()) {
        getMonitor().println(this.i2cTimeLogger.exportLog("I2C transmission time"));
        getMonitor().println(this.flashTimeLogger.exportLog("Burning Flash of time"));
        getMonitor().println(this.resetTimeLogger.exportLog("Reset time"));
      } 
      if (!result)
        return result; 
    } else if (curTransferWay == TransferWay.USBNonNude) {
      result = burnPartsByUSBNonNudeForNandOrSpi(uimonitor, parts, selectedPartCount, hasFlashTypes, fastboot);
      if (!result)
        return result; 
      sendCommandToSetMac();
      String resetCommand = "reset";
      sendCommandNoReturnValue(resetCommand);
    } else {
      result = burnByOtherTransferWay(uimonitor, parts, selectedPartCount, hasFlashTypes, fastboot);
      if (!result)
        return result; 
      sendCommandToSetMac();
      boolean isResetJtag = ((FastbootView.isChipBranchSeries2() || FastbootView.isChipBranchSeries0()) && (curTransferWay == TransferWay.JTAG || curTransferWay == TransferWay.JTAGAndNet || curTransferWay == TransferWay.JTAGAndSerial));
      if (!isResetJtag) {
        String resetCommand = "reset";
        sendCommandNoReturnValue(resetCommand);
      } 
    } 
    infoln("Partition burnt completed!");
    return result;
  }
  
  protected boolean burnByUart2Compress(IProgressMonitor uimonitor, List<PartitionItem> parts, ArrayList<FlashType> hasFlashTypes, PartitionItem fastboot, TransferWay curTransferWay) throws IOException, ReceiveTimeOutException {
    getMonitor().highlight(fastboot);
    uimonitor.setTaskName("Burn boot");
    uimonitor.setTaskName("Prepare to Burn");
    boolean result = prepareToBurn(false, EventType.BURN, hasFlashTypes);
    if (!result) {
      returnError("Failed to Prepare for other partitions!");
      return false;
    } 
    getTempFileDivisionSize(curTransferWay);
    uimonitor.setTaskName("Burning Partitions");
    int partsSize = parts.size();
    for (int index = 0; index < partsSize; index++) {
      PartitionItem partitionItem = parts.get(index);
      boolean isBootargsNeedChanged = false;
      if (!ToolUtils.isCmdLine() && 
        FastbootPreferences.isUpdateBootargsOn())
        isBootargsNeedChanged = true; 
      if (!partitionItem.isChecked()) {
        if (isBootargsNeedChanged && partitionItem.getName().equalsIgnoreCase("bootargs"))
          warningln("Bootargs partition has not been checked, so the automatic update bootargs is not successful!"); 
      } else {
        switchTFTPDownloadHome(partitionItem.getFile());
        getMonitor().highlight(partitionItem);
        long partStart = partitionItem.getStart();
        long partLength = partitionItem.getLength();
        FlashType flashType = partitionItem.getFlashType();
        long startSector = partStart / 512L;
        if (index == partsSize - 1 && partLength == -1L) {
          partLength = getLastPartitionLen(partitionItem, partStart, partLength, flashType, startSector);
          if (partLength == -1L)
            return false; 
        } 
        result = burnPart(SubMonitor.convert(uimonitor, "", 20), partitionItem, parts);
        if (!result)
          return result; 
      } 
    } 
    return result;
  }
  
  protected boolean burnByI2c(IProgressMonitor uimonitor, List<PartitionItem> parts, PartitionItem fastboot, boolean result, ArrayList<FlashType> hasFlashTypes, int selectedPartCount) throws IOException, ReceiveTimeOutException {
    getMonitor().highlight(fastboot);
    uimonitor.setTaskName("Burn boot");
    if (this.burner.getBurnContext().getTransferWay() == TransferWay.I2CBootrom) {
      uimonitor.setTaskName("Prepare to Burn");
      if (fastboot.isChecked()) {
        getMonitor().highlight(fastboot);
        uimonitor.setTaskName("Burn boot");
        result = burnNudeBoard(SubMonitor.convert(uimonitor, "", 100), fastboot, hasFlashTypes, null, true);
        if (!result)
          return false; 
        if (selectedPartCount == 1) {
          if (this.resetTimeLogger != null)
            this.resetTimeLogger.logStart("Start send reset", 0L); 
          String resetCommand = "reset";
          sendCommandNoReturnValue(resetCommand);
          infoln("Partition Burning completed!");
          uimonitor.done();
          if (this.resetTimeLogger != null)
            this.resetTimeLogger.logEnd(); 
          return true;
        } 
        I2cCommand.I2cCloseChannel();
        infoln("\nI2C channels were closed successfully.");
        long startTime = System.currentTimeMillis();
        while (System.currentTimeMillis() - startTime < 5000L);
      } else if (FastbootView.isChipBranchSeries6()) {
        uimonitor.setTaskName("Prepare to Burn");
        result = prepareToBurn(fastboot.isChecked(), EventType.BURN, hasFlashTypes);
        if (!result) {
          returnError("Failed to Prepare for other partitions!");
          return false;
        } 
        I2cCommand.I2cCloseChannel();
        infoln("\nI2C channels were closed successfully.");
      } 
      if (isOnlyBurnBoot(parts, parts.size()))
        return true; 
    } 
    result = burnByUSB2I2C(uimonitor, parts);
    return result;
  }
  
  private boolean burnByUSB2I2C(IProgressMonitor uimonitor, List<PartitionItem> parts) throws IOException, ReceiveTimeOutException {
    boolean result = true;
    int partsSize = parts.size();
    uimonitor.setTaskName("Burning Partitions");
    if (!I2cCommand.I2cOpenChannel())
      return false; 
    infoln("\nI2C channels open success, the bit rate is " + I2cCommand.getBaudRate() + ", device address is 0x" + 
        Integer.toHexString(I2cCommand.getDeviceAddress()));
    uimonitor.setTaskName("Burning Partitions");
    if (!getLastPartitionLen(parts, partsSize))
      return false; 
    int i2cDelay = FastbootPreferences.getInt("vendor.burn.i2c.delay");
    uimonitor.setTaskName("Burning Partitions");
    int startIndex = (this.burner.getBurnContext().getTransferWay() == TransferWay.Usb2I2c) ? 0 : 1;
    for (int index = startIndex; index < partsSize; index++) {
      if (uimonitor.isCanceled()) {
        boolean handleCancel = handleCancel();
        if (handleCancel) {
          closeI2cAndDeleteTempFile(null, null);
          return false;
        } 
      } 
      PartitionItem partitionItem = parts.get(index);
      getMonitor().highlight(partitionItem);
      File file = partitionItem.getFile();
      if (partitionItem.isChecked()) {
        boolean fileVaild = (file == null || !file.exists());
        if (fileVaild) {
          printWarnToPartition(index, partitionItem);
        } else {
          SubMonitor subProgressMonitor = SubMonitor.convert(uimonitor, "", 100);
          long partStart = partitionItem.getStart();
          String name = partitionItem.getName();
          File partFile = copyComFileToToolPath(uimonitor, file);
          if (partFile == null)
            return false; 
          File compressFile = GZipUtils.compress(uimonitor, partFile, false, true);
          if (compressFile == null) {
            returnError(NLS.bind(Messages.ERROR_INFO_DISK_SPACE_LOW, partFile));
            return false;
          } 
          long compressFileLen = compressFile.length();
          BufferedRandomAccessFile inFile = null;
          try {
            inFile = new BufferedRandomAccessFile(compressFile, "r");
            result = I2cCommand.sendUpgradeStart((int)compressFileLen, true);
            if (!result) {
              closeI2cAndDeleteTempFile(compressFile, partFile);
              return result;
            } 
            result = sendUpgradeRun(uimonitor, result, i2cDelay, subProgressMonitor, partStart, name, partFile, compressFile, compressFileLen, inFile);
            if (!result) {
              closeI2cAndDeleteTempFile(compressFile, partFile);
              return result;
            } 
          } catch (Exception exception) {
          
          } finally {
            if (inFile != null)
              inFile.close(); 
          } 
          receivedln(" ---- 100%");
          ChipID chipID = partitionItem.getChipID();
          result = I2cCommand.sendUpgradeFinish((int)partStart, chipID);
          if (!result) {
            closeI2cAndDeleteTempFile(compressFile, partFile);
            return result;
          } 
          int waitWriteTimeout = FastbootPreferences.getI2cTimeout();
          result = checkUpgradeStatus(file, waitWriteTimeout, i2cDelay);
          if (!result) {
            closeI2cAndDeleteTempFile(compressFile, partFile);
            return result;
          } 
          FileUtils.deleteFile(compressFile);
          FileUtils.deleteFile(partFile);
        } 
      } 
    } 
    sendCommandToSetMac();
    closeI2cAndDeleteTempFile(null, null);
    return result;
  }
  
  private boolean sendUpgradeRun(IProgressMonitor uimonitor, boolean result, int i2cDelay, SubMonitor subProgressMonitor, long partStart, String name, File partFile, File compressFile, long compressFileLen, BufferedRandomAccessFile inFile) throws IOException {
    int dataCount = (int)((compressFileLen + 124L - 1L) / 124L);
    int dataSpace = (int)compressFileLen % 124;
    printI2cLog(uimonitor, subProgressMonitor, partStart, name, compressFile, compressFileLen, dataCount);
    int totalOffset = 0;
    double rateLast = 0.0D;
    int total = (int)compressFileLen;
    byte[] data = new byte[124];
    int upgradeIndex = 0;
    if (dataCount == 0) {
      int read = inFile.read(data, 0, data.length);
      if (read != data.length)
        return false; 
      result = I2cCommand.sendUpgradeRun(upgradeIndex, data);
      if (!result) {
        closeI2cAndDeleteTempFile(compressFile, partFile);
        return result;
      } 
      upgradeIndex++;
    } else {
      for (int j = 0; j < dataCount; j++) {
        if (uimonitor.isCanceled()) {
          boolean handleCancel = handleCancel();
          if (handleCancel) {
            closeI2cAndDeleteTempFile(compressFile, partFile);
            return false;
          } 
        } 
        int len = 0;
        if (j == dataCount - 1) {
          if (dataSpace == 0) {
            len = 124;
          } else {
            len = dataSpace;
            data = new byte[124];
          } 
        } else {
          len = 124;
        } 
        int read = inFile.read(data, 0, len);
        if (read != len)
          return false; 
        if (upgradeIndex == 256)
          upgradeIndex = 0; 
        result = I2cCommand.sendUpgradeRun(upgradeIndex, data);
        if (!result) {
          closeI2cAndDeleteTempFile(compressFile, partFile);
          return result;
        } 
        if (upgradeIndex != 0 && upgradeIndex % 10 == 0) {
          int status = I2cCommand.sendGetUpgradeStatus(i2cDelay, monitor);
          if (status == -1) {
            closeI2cAndDeleteTempFile(compressFile, partFile);
            returnError(Messages.PartitionBurnerWork_I2c_Upgrade_Fail);
            return false;
          } 
        } 
        upgradeIndex++;
        totalOffset += len;
        received("#");
        double temp = totalOffset * 100.0D / total;
        boolean needPrint = (temp - rateLast >= 1.0D && (int)temp != 100);
        if (needPrint) {
          receivedln(" ---- " + (int)temp + "%");
          rateLast = temp;
        } 
        subProgressMonitor.worked(1);
      } 
    } 
    return result;
  }
  
  private void printI2cLog(IProgressMonitor uimonitor, SubMonitor subProgressMonitor, long partStart, String name, File compressFile, long compressFileLen, int dataCount) {
    uimonitor.setTaskName("Downloading File");
    subProgressMonitor.beginTask("Burn the " + name, dataCount);
    subProgressMonitor.setTaskName("Downloading File");
    infoln("\n[Download File by I2c]");
    try {
      infoln("file = " + compressFile.getCanonicalPath());
    } catch (IOException iOException) {}
    infoln("address = " + partStart);
    infoln("fileLength = " + compressFileLen);
  }
  
  private void printWarnToPartition(int index, PartitionItem partitionItem) {
    if (partitionItem.getName() != null) {
      warningln(NLS.bind(Messages.PartitionBurnerWork_I2c_Not_Exist, partitionItem.getName()));
    } else {
      warningln(NLS.bind(Messages.PartitionBurnerWork_I2c_Not_Exist, "The Line " + index + 1 + " file"));
    } 
  }
  
  private boolean getLastPartitionLen(List<PartitionItem> parts, int partsSize) {
    PartitionItem lastPartitionItem = parts.get(partsSize - 1);
    if (lastPartitionItem.isChecked()) {
      long lastPartitionLen = lastPartitionItem.getLength();
      if (lastPartitionLen == -1L && 
        !checkLastPartition(parts)) {
        returnError(Messages.PartitionBurnerWork_GetLastPartitionLen_Fail);
        closeI2cAndDeleteTempFile(null, null);
        return false;
      } 
    } 
    return true;
  }
  
  private boolean isOnlyBurnBoot(List<PartitionItem> parts, int partsSize) {
    boolean onlyBootChecked = true;
    for (int index = 1; index < partsSize; index++) {
      PartitionItem partitionItem = parts.get(index);
      getMonitor().highlight(partitionItem);
      if (partitionItem.isChecked()) {
        onlyBootChecked = false;
        break;
      } 
    } 
    if (onlyBootChecked)
      return true; 
    return false;
  }
  
  private boolean burnByOtherTransferWay(IProgressMonitor uimonitor, List<PartitionItem> parts, int selectedPartCount, ArrayList<FlashType> hasFlashTypes, PartitionItem fastboot) throws IOException, ReceiveTimeOutException {
    SubMonitor subProgressMonitor = SubMonitor.convert(uimonitor, "", 100);
    registerJnaFunction(uimonitor, 100);
    if (fastboot.isChecked()) {
      getMonitor().highlight(fastboot);
      uimonitor.setTaskName("Burn boot");
      boolean bool = burnNudeBoard(subProgressMonitor, fastboot, hasFlashTypes, parts, true);
      if (!bool)
        return false; 
      if (selectedPartCount == 1) {
        infoln("Partition Burning completed!");
        uimonitor.done();
        return true;
      } 
    } 
    uimonitor.setTaskName("Prepare to Burn");
    boolean result = prepareToBurn(fastboot.isChecked(), EventType.BURN, hasFlashTypes);
    if (!result) {
      returnError("Failed to Prepare for other partitions!");
      return false;
    } 
    uimonitor.setTaskName("Burning Partitions");
    int partsSize = parts.size();
    for (int index = 1; index < partsSize; index++) {
      PartitionItem partitionItem = parts.get(index);
      boolean isBootargsNeedChanged = false;
      if (!ToolUtils.isCmdLine() && 
        FastbootPreferences.isUpdateBootargsOn())
        isBootargsNeedChanged = true; 
      if (!partitionItem.isChecked()) {
        if (isBootargsNeedChanged && partitionItem.getName().equalsIgnoreCase("bootargs"))
          warningln("Bootargs partition has not been checked, so the automatic update bootargs is not successful!"); 
      } else {
        switchTFTPDownloadHome(partitionItem.getFile());
        getMonitor().highlight(partitionItem);
        long partStart = partitionItem.getStart();
        long partLength = partitionItem.getLength();
        FlashType flashType = partitionItem.getFlashType();
        long startSector = partStart / 512L;
        if (index == partsSize - 1 && partLength == -1L) {
          partLength = getLastPartitionLen(partitionItem, partStart, partLength, flashType, startSector);
          if (partLength == -1L)
            return false; 
        } 
        result = burnPart(subProgressMonitor, partitionItem, parts);
        if (!result)
          return result; 
      } 
    } 
    return result;
  }
  
  protected boolean burnEachPartForNandOrSpi(IProgressMonitor uimonitor, List<PartitionItem> parts, int index) throws IOException, ReceiveTimeOutException {
    int partsSize = parts.size();
    PartitionItem partitionItem = parts.get(index);
    boolean isBootargsNeedChanged = false;
    if (!ToolUtils.isCmdLine() && 
      FastbootPreferences.isUpdateBootargsOn())
      isBootargsNeedChanged = true; 
    if (!partitionItem.isChecked()) {
      if (isBootargsNeedChanged && partitionItem.getName().equalsIgnoreCase("bootargs"))
        warningln("Bootargs partition has not been checked, so the automatic update bootargs is not successful!"); 
      return true;
    } 
    switchTFTPDownloadHome(partitionItem.getFile());
    getMonitor().highlight(partitionItem);
    long partStart = partitionItem.getStart();
    long partLength = partitionItem.getLength();
    FlashType flashType = partitionItem.getFlashType();
    long startSector = partStart / 512L;
    if (index == partsSize - 1 && partLength == -1L) {
      partLength = getLastPartitionLen(partitionItem, partStart, partLength, flashType, startSector);
      if (partLength == -1L)
        return false; 
    } 
    return burnPart(SubMonitor.convert(uimonitor, "", 100), partitionItem, parts);
  }
  
  private boolean burnPartsByUSBNonNudeForNandOrSpi(IProgressMonitor uimonitor, List<PartitionItem> parts, int selectedPartCount, ArrayList<FlashType> hasFlashTypes, PartitionItem fastboot) throws IOException, ReceiveTimeOutException {
    boolean result = false;
    result = bootromByNonNude(uimonitor);
    if (!result)
      return false; 
    if (fastboot.isChecked()) {
      result = downloadBootByUSB(uimonitor, fastboot.getFile(), this.burner.getBurnContext().getTransferWay());
      if (!result)
        return false; 
      burnbootByUSBNonNude(SubMonitor.convert(uimonitor, "", 100), selectedPartCount, hasFlashTypes, fastboot);
    } else if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
      loadBlockSizes(hasFlashTypes, fastboot.getChipID(), false);
    } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
      loadBlockSizes(hasFlashTypes, fastboot.getChipID(), false);
      warningln("Warning: Failed to load version info!");
    } else {
      returnError("Failed to load version info!");
      return false;
    } 
    uimonitor.setTaskName("Burning Partitions");
    int partsSize = parts.size();
    for (int index = 1; index < partsSize; index++) {
      PartitionItem partitionItem = parts.get(index);
      boolean isBootargsNeedChanged = false;
      if (!ToolUtils.isCmdLine() && 
        FastbootPreferences.isUpdateBootargsOn())
        isBootargsNeedChanged = true; 
      if (!partitionItem.isChecked()) {
        if (isBootargsNeedChanged && partitionItem.getName().equalsIgnoreCase("bootargs"))
          warningln("Bootargs partition has not been checked, so the automatic update bootargs is not successful!"); 
      } else {
        getMonitor().highlight(partitionItem);
        long partStart = partitionItem.getStart();
        long partLength = partitionItem.getLength();
        FlashType flashType = partitionItem.getFlashType();
        long startSector = partStart / 512L;
        if (index == partsSize - 1 && partLength == -1L) {
          partLength = getLastPartitionLen(partitionItem, partStart, partLength, flashType, startSector);
          if (partLength == -1L)
            return false; 
        } 
        result = burnPart(SubMonitor.convert(uimonitor, "", 100), partitionItem, parts);
        if (!result)
          return result; 
      } 
    } 
    return result;
  }
  
  protected boolean burnbootForNandOrSpi(IProgressMonitor uimonitor, List<PartitionItem> parts, int selectedPartCount, ArrayList<FlashType> hasFlashTypes, PartitionItem fastboot, TransferWay curTransferWay) throws IOException, ReceiveTimeOutException {
    boolean result = false;
    if (curTransferWay == TransferWay.USBNonNude) {
      if (FastbootView.isChipBranchSeries4()) {
        boolean openUsbResult = CallLibUSBWorker.openUsb(uimonitor, 30000L, curTransferWay, 
            BurnConfig.getUsbSerialNumber());
        if (!openUsbResult) {
          returnError("Failed to open usb!");
          infoln("Failed to open usb!");
          return false;
        } 
      } else {
        result = bootromByNonNude(uimonitor);
        if (!result)
          return false; 
      } 
      if (fastboot.isChecked()) {
        result = downloadBootByUSB(uimonitor, fastboot.getFile(), this.burner.getBurnContext().getTransferWay());
        if (!result)
          return false; 
        burnbootByUSBNonNude(SubMonitor.convert(uimonitor, "", 100), selectedPartCount, hasFlashTypes, fastboot);
      } else if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
        loadBlockSizes(hasFlashTypes, fastboot.getChipID(), false);
      } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
        loadBlockSizes(hasFlashTypes, fastboot.getChipID(), false);
        warningln("Warning: Failed to load version info!");
      } else {
        returnError("Failed to load version info!");
        return false;
      } 
    } else {
      if (fastboot.isChecked()) {
        getMonitor().highlight(fastboot);
        uimonitor.setTaskName("Burn boot");
        result = burnNudeBoard(SubMonitor.convert(uimonitor, "", 100), fastboot, hasFlashTypes, parts, true);
        if (!result)
          return false; 
        if (selectedPartCount == 1) {
          infoln("Partition Burning completed!");
          uimonitor.done();
          return true;
        } 
      } 
      uimonitor.setTaskName("Prepare to Burn");
      result = prepareToBurn(fastboot.isChecked(), EventType.BURN, hasFlashTypes);
      if (!result) {
        returnError("Failed to Prepare for other partitions!");
        return false;
      } 
    } 
    return result;
  }
  
  private boolean burnbootByUSBNonNude(SubMonitor subProgressMonitor, int selectedPartCount, ArrayList<FlashType> hasFlashTypes, PartitionItem fastboot) throws ReceiveTimeOutException, IOException {
    getMonitor().highlight(fastboot);
    this.uimonitor.setTaskName("Burn boot");
    FlashType flashType = fastboot.getFlashType();
    subProgressMonitor.setTaskName("Loading Version Info");
    if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
      FlashType bootMode = loadBootMode();
      if (bootMode != null && flashType != bootMode) {
        returnError(NLS.bind(Messages.ERROR_INFO_Invalid_Mode_And_RealMode, bootMode.name()));
        return false;
      } 
      subProgressMonitor.worked(5);
      subProgressMonitor.setTaskName("Loading Block Size");
      loadBlockSizes(hasFlashTypes, fastboot.getChipID(), false);
      subProgressMonitor.worked(5);
    } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
      subProgressMonitor.worked(5);
      subProgressMonitor.setTaskName("Loading Block Size");
      loadBlockSizes(hasFlashTypes, fastboot.getChipID(), false);
      subProgressMonitor.worked(5);
      warningln("Warning: Failed to load version info!");
    } else {
      returnError("Failed to load version info!");
      return false;
    } 
    long uLength = fastboot.getLength();
    if (FlashType.spi == flashType && 0L != getSpiBlockSize() && 
      uLength % getSpiBlockSize() != 0L) {
      String error = "Please check partition length of '" + fastboot.getName() + "'!(should be " + getSpiBlockSize() + " * n)";
      returnError(error);
      return false;
    } 
    long blockSize = 0L;
    if (FlashType.spi == flashType) {
      blockSize = getSpiBlockSize();
    } else {
      blockSize = getNandBlockSize();
      if (isPage())
        blockSize = getPageSize(); 
    } 
    if (0L == blockSize) {
      returnError("Invalid " + flashType.name() + " flash block size!");
      return false;
    } 
    if (uLength % blockSize != 0L) {
      String error = "Please check partition length of '" + fastboot.getName() + "'!(should be " + blockSize + " * n)";
      returnError(error);
      return false;
    } 
    long fileSize = fastboot.getFile().length();
    long writeLength = 0L;
    if (fileSize % blockSize != 0L) {
      writeLength = blockSize * (fileSize / blockSize + 1L);
    } else {
      writeLength = fileSize;
    } 
    if (!bootBurn(SubMonitor.convert(this.uimonitor, "", 20), flashType, 
        Long.toHexString(uLength), Long.toHexString(writeLength), fastboot.getChipID())) {
      returnError("Failed to burn fastboot partition!");
      return false;
    } 
    if (selectedPartCount == 1) {
      infoln("Partition Burning completed!");
      this.uimonitor.done();
      return true;
    } 
    return true;
  }
  
  public boolean prepareToBurn(boolean bootLoaded, EventType eventType, ArrayList<FlashType> hasFlashTypes) throws IOException, ReceiveTimeOutException {
    boolean result = true;
    if (!bootLoaded) {
      String message = "Failed to Download Boot!";
      if (eventType == EventType.ERASE)
        message = Messages.BurnComposite_Erase_Message_Failed; 
      if (((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportBurnByLibBootrom()) {
        if (!programmerDownload(null, ((AbstractBurner)this.burner).getProgrammerFile())) {
          returnError(message);
          return false;
        } 
      } else {
        if (this.burner.getBurnContext().getPlatformType() == PlatformType.NonCA) {
          result = bootDownload(null);
        } else {
          result = programmerDownload(null, ((AbstractBurner)this.burner).getProgrammerFile());
        } 
        if (!result) {
          returnError(message);
          return false;
        } 
      } 
      if (!startFastbootUdpTransferWay(this.uimonitor, this.burner.getBurnContext().getTransferWay()))
        return false; 
      if (!startFastbootUsbTransferWay(this.uimonitor, this.burner.getBurnContext().getTransferWay()))
        return false; 
      loadBlockSizes(hasFlashTypes, ChipID.chip0, false);
    } 
    if (EventType.BURN == eventType || EventType.UPLOAD == eventType) {
      TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
      if (isNeedSetupNet(transferWay))
        return (result && setupNetwork(transferWay)); 
      return result;
    } 
    return result;
  }
  
  public boolean burnNudeBoard(SubMonitor subProgressMonitor, PartitionItem first, ArrayList<FlashType> hasFlashTypes, List<PartitionItem> parts, boolean isFastboot) throws IOException, ReceiveTimeOutException {
    try {
      subProgressMonitor.beginTask("", 100);
      subProgressMonitor.setTaskName("Downloading Boot");
      if (this.i2cTimeLogger != null && first.getFile() != null)
        this.i2cTimeLogger.logStart("Download boot via I2C", first.getFile().length()); 
      setFlashType(first.getFlashType());
      String bootPath = first.getFile().getPath();
      if (((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportBurnByLibBootrom()) {
        if (!programmerDownload(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 75), bootPath, ((AbstractBurner)this.burner)
            .getProgrammerFile(), parts))
          return false; 
      } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.NonCA) {
        if (!bootDownload(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 75), bootPath))
          return false; 
      } else if (!programmerDownload(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 75), bootPath, ((AbstractBurner)this.burner)
          .getProgrammerFile(), parts)) {
        return false;
      } 
      this.uimonitor.setTaskName("Check SBL");
      boolean result = burnSBL(this.uimonitor, parts, ((AbstractBurner)this.burner).getProgrammerFile());
      if (!result)
        return result; 
      FlashType flashType = first.getFlashType();
      subProgressMonitor.setTaskName("Loading Version Info");
      if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
        FlashType bootMode = loadBootMode();
        if (bootMode != null && flashType != bootMode) {
          returnError(NLS.bind(Messages.ERROR_INFO_Invalid_Mode_And_RealMode, bootMode.name()));
          return false;
        } 
        subProgressMonitor.worked(5);
        subProgressMonitor.setTaskName("Loading Block Size");
        loadBlockSizes(hasFlashTypes, first.getChipID(), isFastboot);
        subProgressMonitor.worked(5);
      } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
        subProgressMonitor.worked(5);
        subProgressMonitor.setTaskName("Loading Block Size");
        loadBlockSizes(hasFlashTypes, first.getChipID(), isFastboot);
        subProgressMonitor.worked(5);
        warningln("Warning: Failed to load version info!");
      } else {
        returnError("Failed to load version info!");
        return false;
      } 
      long uLength = first.getLength();
      long blockSize = 0L;
      if (FlashType.spi == flashType) {
        blockSize = getSpiBlockSize();
      } else if (FlashType.nand == flashType) {
        blockSize = getNandBlockSize();
        if (isPage() && !isFastboot)
          blockSize = getPageSize(); 
      } 
      if (0L == blockSize) {
        returnError("Invalid " + flashType.name() + " flash block size!");
        return false;
      } 
      if (uLength % blockSize != 0L) {
        String error = "Please check partition length of '" + first.getName() + "'!(should be " + blockSize + " * n)";
        returnError(error);
        return false;
      } 
      long fileSize = first.getFile().length();
      long writeLength = 0L;
      if (fileSize % blockSize != 0L) {
        writeLength = blockSize * (fileSize / blockSize + 1L);
      } else {
        writeLength = fileSize;
      } 
      if (this.i2cTimeLogger != null)
        this.i2cTimeLogger.logEnd(); 
      if (this.flashTimeLogger != null)
        this.flashTimeLogger.logStart(first.getFile().getName(), writeLength); 
      if (!burnFastboot(subProgressMonitor, first, writeLength, parts))
        return false; 
      if (this.flashTimeLogger != null)
        this.flashTimeLogger.logEnd(); 
      return true;
    } finally {
      subProgressMonitor.done();
    } 
  }
  
  private boolean burnFastboot(SubMonitor subProgressMonitor, PartitionItem first, long writeLength, List<PartitionItem> parts) throws IOException, ReceiveTimeOutException {
    if (FastbootView.isSupportSerial() || FastbootView.isChipBranchSeries10()) {
      TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
      if (isNeedSetupNet(transferWay)) {
        if (!setupNetwork(transferWay)) {
          returnError("Failed to set net work!");
          return false;
        } 
        switchTFTPDownloadHome(first.getFile());
      } 
      if (!burnPart(subProgressMonitor, first, parts)) {
        returnError("Failed to burn fastboot!");
        return false;
      } 
      return true;
    } 
    FlashType flashType = first.getFlashType();
    long uLength = first.getLength();
    if (!bootBurn(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 15), flashType, Long.toHexString(uLength), 
        Long.toHexString(writeLength), first.getChipID())) {
      returnError("Failed to burn fastboot partition!");
      return false;
    } 
    return true;
  }
  
  private boolean burnPart(SubMonitor subProgressMonitor, PartitionItem partitionItem, List<PartitionItem> parts) throws IOException, ReceiveTimeOutException {
    try {
      String partitionName = (partitionItem.getName() == null) ? "" : partitionItem.getName();
      subProgressMonitor.beginTask("", 100);
      subProgressMonitor.setTaskName("Partition " + partitionName);
      getMonitor().highlight(partitionItem);
      long partStart = 0L;
      partStart = partitionItem.getStart();
      long partitionLength = partitionItem.getLength();
      long writeLenght = partitionLength;
      FlashType flashType = partitionItem.getFlashType();
      FileSystem fileSystem = partitionItem.getFileSystem();
      File itemFile = null;
      boolean isBootargsNeedChanged = false;
      if (!ToolUtils.isCmdLine() && 
        FastbootPreferences.isUpdateBootargsOn())
        isBootargsNeedChanged = true; 
      if (isBootargsNeedChanged && partitionItem.getName().equalsIgnoreCase("bootargs") && partitionItem
        .getFile() != null) {
        this.tmpBootargsString = getBootargsPartitionString(parts, "soct_sfc", "soct_nand");
        if (this.tmpBootargsString == null)
          return false; 
        itemFile = prepareReplacedBootargs(partitionItem.getFile(), this.tmpBootargsString);
      } else {
        itemFile = partitionItem.getFile();
      } 
      subProgressMonitor.subTask("Prepare and Validate");
      if (getSpiBlockSize() == 0L && getNandBlockSize() == 0L)
        loadBlockSizes(FlashType.getCurFlashTypes(parts), partitionItem.getChipID(), false); 
      long blockSize = getNandBlockSize();
      if (isPage())
        blockSize = getPageSize(); 
      if (FlashType.spi == flashType)
        blockSize = getSpiBlockSize(); 
      if (0L == blockSize) {
        returnError("Invalid " + flashType.name() + " flash block size!");
        return false;
      } 
      long fileSize = 0L;
      if (itemFile != null && itemFile.exists() && itemFile.isFile()) {
        fileSize = itemFile.length();
      } else {
        fileSize = 0L;
      } 
      if (fileSize % blockSize == 0L) {
        writeLenght = fileSize;
      } else {
        writeLenght = fileSize / blockSize * blockSize + blockSize;
      } 
      if (partitionLength % blockSize != 0L) {
        String error = "Please check partition length of '" + partitionItem.getName() + "'!(should be " + blockSize + " * n)";
        returnError(error);
        return false;
      } 
      subProgressMonitor.worked(2);
      long filePartSize = ToolUtils.getSplitFileSize();
      if (fileSystem == FileSystem.yaffs) {
        long[] yaffsPageandOOBSize = getYaffsPageandOOBSize();
        long ulPageSize = yaffsPageandOOBSize[0];
        long ulOOBSize = yaffsPageandOOBSize[1];
        if (0L == ulPageSize || 0L == ulOOBSize) {
          returnError("Failed to get yaffs block size in Partition " + partitionName + " !");
          return false;
        } 
        writeLenght = fileSize;
        filePartSize = filePartSize / blockSize * (blockSize + blockSize / ulPageSize * ulOOBSize);
      } 
      boolean result = true;
      if (writeLenght > filePartSize) {
        if (itemFile == null) {
          returnError("Partition " + partitionName + " burning failed!");
          return false;
        } 
        if (this.burner instanceof com.vendor.burn.model.devices.burners.CommonBurner) {
          com.vendor.burn.serial.workers.PartitionBurnerWorker worker = new com.vendor.burn.serial.workers.PartitionBurnerWorker(this.burner);
          worker.setProgressMonitor(this.uimonitor);
          this.tftpTimeLogger = new TimeLogger();
          this.burnTimeLogger = new TimeLogger();
          result = worker.burnLargeFile(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 98), itemFile, fileSystem, flashType, filePartSize, partStart, partitionLength, writeLenght, partitionItem
              
              .getChipID());
        } else {
          result = burnLargeFile(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 98), itemFile, fileSystem, flashType, filePartSize, partStart, partitionLength, writeLenght, partitionItem
              .getChipID());
        } 
      } else if (this.burner instanceof com.vendor.burn.model.devices.burners.CommonBurner) {
        com.vendor.burn.serial.workers.PartitionBurnerWorker worker = new com.vendor.burn.serial.workers.PartitionBurnerWorker(this.burner);
        worker.setProgressMonitor(this.uimonitor);
        this.tftpTimeLogger = new TimeLogger();
        this.burnTimeLogger = new TimeLogger();
        result = worker.burnSmallFile(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 98), itemFile, fileSystem, flashType, partStart, partitionLength, writeLenght, partitionItem
            .getChipID());
      } else {
        result = burnSmallFile(SubMonitor.convert((IProgressMonitor)subProgressMonitor, "", 98), itemFile, fileSystem, flashType, partStart, partitionLength, writeLenght, partitionItem
            .getChipID());
      } 
      if (!result) {
        returnError("Partition " + partitionName + " burning failed!");
        return false;
      } 
      infoln("Partition " + partitionName + " burnt successfully!");
      if (itemFile != partitionItem.getFile() && 
        !itemFile.delete())
        itemFile.deleteOnExit(); 
      return result;
    } finally {
      subProgressMonitor.done();
    } 
  }
  
  private PartitionItem getPartLengthAndPartFile(List<PartitionItem> partitionItemList, PartitionItem item, int times, FileInputStream inputStream, long adjustedTotalWriteLength) throws IOException, ReceiveTimeOutException {
    int partLength = 0;
    File partfile = null;
    String toolPath = "";
    String partFilePath = "";
    FileSystem fileSystem = item.getFileSystem();
    File file = item.getFile();
    long filePartSize = item.getLength();
    if (fileSystem == FileSystem.yaffs) {
      PartitionItem partitionItem = partitionItemList.get(times);
      partLength = (int)partitionItem.getLength();
      partfile = partitionItem.getFile();
    } else {
      if (getBurner().getBurnContext().isTftpOccupied()) {
        partFilePath = file.getPath() + ".part" + file.getPath();
      } else {
        toolPath = ToolUtils.getInstallPath();
        partFilePath = toolPath + toolPath + File.separator + ".part" + file.getName();
      } 
      partfile = new File(partFilePath);
      boolean isCreate = FileUtils.isCreateNewFile(partfile);
      if (!isCreate) {
        returnError(NLS.bind(Messages.AddressBurnerWorker_PartFile_Create_Failed, partfile
              .getPath()));
        return new PartitionItem();
      } 
      if (toolPath != null)
        switchTFTPDownloadHome(partfile.getParentFile()); 
      partLength = writeBufferToPartFile(filePartSize, inputStream, true, partfile, partLength);
      if (partLength < filePartSize)
        if (adjustedTotalWriteLength % filePartSize == 0L) {
          partLength = (int)filePartSize;
        } else {
          partLength = (int)(adjustedTotalWriteLength % filePartSize);
        }  
    } 
    item.setFile(partfile);
    item.setLength(partLength);
    return item;
  }
  
  protected boolean burnLargeFile(SubMonitor subProgressMonitor, File file, FileSystem fileSystem, FlashType flashType, long filePartSize, long start, long eraseLength, long adjustedTotalWriteLength, ChipID chipID) throws IOException, ReceiveTimeOutException {
    int tasks = (int)(file.length() / filePartSize + 1L);
    subProgressMonitor.beginTask("Burning " + file.getName(), tasks * 100 + 5);
    FileInputStream inputStream = null;
    boolean result = true;
    try {
      if (result && flashType == FlashType.spi)
        result = sendCommand((SerialCommand)SFCommand.getInstance(chipID)); 
      if (result) {
        EraseCommand eraseCommand = EraseCommand.erase(flashType, Long.toHexString(start), Long.toHexString(eraseLength));
        TransferWay transferWay = getBurner().getBurnContext().getTransferWay();
        if (FlashType.fuzzyCompareFlash(flashType, FlashType.emmc) && (transferWay == TransferWay.UsbAndSerial || 
          FastbootView.isSupportUsbNonNude(transferWay))) {
          infoln("\nSend command:\t " + eraseCommand.getCommand());
          result = CallLibUSBWorker.eraseFlashByUSB((int)start, (int)eraseLength, eraseCommand
              .getTimeout());
        } else {
          result = sendCommand((SerialCommand)eraseCommand);
        } 
        subProgressMonitor.worked(5);
      } 
      if (file.exists() && file.isFile())
        inputStream = new FileInputStream(file); 
      long address = start;
      List<PartitionItem> partitionItemList = new ArrayList<>();
      int times = 0;
      if (fileSystem == FileSystem.yaffs) {
        PartitionItem item = new PartitionItem();
        item.setFile(file);
        item.setFileSystem(fileSystem);
        item.setStart(address);
        item.setLength(eraseLength);
        partitionItemList = SparseFileAgent.getYaffsFileXmlList(item, getYaffsPageandOOBSize(), 
            getBurner().getBurnContext().isTftpOccupied());
      } 
      long totalLength = file.length();
      while ((result && totalLength > 0L) || times < partitionItemList.size()) {
        File partfile = null;
        int partLength = 0;
        try {
          PartitionItem item = new PartitionItem();
          item.setFile(file);
          item.setFileSystem(fileSystem);
          item.setLength(filePartSize);
          item = getPartLengthAndPartFile(partitionItemList, item, times, inputStream, adjustedTotalWriteLength);
          partfile = item.getFile();
          partLength = (int)item.getLength();
          if (partfile == null)
            return false; 
          totalLength -= partLength;
          subProgressMonitor.worked(5);
          String ddrAddress = getHexStartAddress();
          String unzipAddr = Long.toHexString(tempFileDivisionSize + getHexStartAddressForInt());
          if (getBurner().getBurnContext().getTransferWay() == TransferWay.Uart2Compress)
            ddrAddress = unzipAddr; 
          WriteCommand writeCommand = WriteCommand.write(flashType, ddrAddress, Long.toHexString(address), 
              Long.toHexString(partLength));
          if (fileSystem == FileSystem.yaffs)
            if (0 == times) {
              writeCommand = WriteCommand.writeYaffs(flashType, ddrAddress, Long.toHexString(address), 
                  Long.toHexString(partLength));
            } else {
              writeCommand = WriteCommand.writeYaffsuc(flashType, ddrAddress, Long.toHexString(address), 
                  Long.toHexString(partLength));
            }  
          subProgressMonitor.setTaskName("Sending MW command.");
          MWCommand memsetCommand = MWCommand.mwAddress(getHexStartAddress(), Long.toHexString(partLength));
          result = sendCommand((SerialCommand)memsetCommand);
          subProgressMonitor.worked(2);
          long readFileLen = 0L;
          XmodelParameter xmodelPar = new XmodelParameter(file.length(), readFileLen);
          result = downloadFileByTrans(subProgressMonitor, result, xmodelPar, partfile, 90);
          if (!result)
            return result; 
          readFileLen += partfile.length();
          if (result) {
            if (getBurner().getBurnContext().getTransferWay() == TransferWay.Uart2Compress) {
              UnZipCommand unZipCommand = UnZipCommand.unzip(getHexStartAddress(), unzipAddr);
              result = sendCommand(unZipCommand.toString(), unZipCommand
                  .calculateTimeOut(Long.toHexString(tempFileDivisionSize)));
            } 
            this.burnTimeLogger.logStart(partfile.getName(), partLength);
            result = sendCommand((SerialCommand)writeCommand);
            this.burnTimeLogger.logEnd();
          } 
          int burnCount = 1;
          while (!result) {
            String returnString = getCurReceived();
            if (returnString.contains("WARNING: block is marked as bad")) {
              warningln("A block is marked as bad block, burning the file again,the current is the " + burnCount + " times");
              subProgressMonitor.setTaskName("Writing...");
              this.burnTimeLogger.logStart(partfile.getName(), partLength);
              result = sendCommand((SerialCommand)writeCommand);
              this.burnTimeLogger.logEnd();
              burnCount++;
            } 
          } 
          if (fileSystem == FileSystem.yaffs || FlashType.nand == flashType) {
            long realBurnLength = getRealBurnLength(eraseLength);
            if (realBurnLength == -1L) {
              returnError("Failed to get real burt length.");
              return false;
            } 
            if (realBurnLength == 0L) {
              infoln("Warning: BurnTool didn't receive the real burning length from the board, so please try to use the release version of the boot");
              address += partLength;
            } else {
              address += realBurnLength;
            } 
          } else {
            address += partLength;
          } 
          if (start + eraseLength < address) {
            returnError(BurntLengthGreaterEraseLength);
            return false;
          } 
          subProgressMonitor.worked(5);
          times++;
          if (!result) {
            if (partfile != null && partfile.exists())
              FileUtils.deleteFile(partfile); 
            break;
          } 
        } finally {
          if (partfile != null && partfile.exists())
            FileUtils.deleteFile(partfile); 
        } 
      } 
      return result;
    } finally {
      if (inputStream != null)
        try {
          inputStream.close();
        } catch (Exception exception) {} 
      subProgressMonitor.done();
    } 
  }
  
  protected boolean burnSmallFile(SubMonitor subProgressMonitor, File file, FileSystem fileSystem, FlashType flashType, long address, long eraseLength, long writeLength, ChipID chipID) throws IOException, ReceiveTimeOutException {
    if (subProgressMonitor == null)
      return false; 
    String name = (file != null) ? file.getName() : "";
    subProgressMonitor.beginTask("Burning " + name, 100);
    try {
      boolean result = true;
      String hexAddress = Long.toHexString(address);
      String hexWriteLength = Long.toHexString(writeLength);
      String hexEraseLength = Long.toHexString(eraseLength);
      EraseCommand eraseCommand = EraseCommand.erase(flashType, hexAddress, hexEraseLength);
      String unzipAddr = Long.toHexString(tempFileDivisionSize + getHexStartAddressForInt());
      String ddrAddress = (getBurner().getBurnContext().getTransferWay() == TransferWay.Uart2Compress) ? unzipAddr : getHexStartAddress();
      WriteCommand writeCommand = (fileSystem == FileSystem.yaffs) ? WriteCommand.writeYaffs(flashType, ddrAddress, hexAddress, hexWriteLength) : WriteCommand.write(flashType, ddrAddress, hexAddress, hexWriteLength);
      boolean validFile = (file != null && file.exists() && file.isFile());
      if (validFile) {
        subProgressMonitor.setTaskName("Sending MW command.");
        MWCommand memsetCommand = MWCommand.mwAddress(getHexStartAddress(), hexWriteLength);
        result = sendCommand((SerialCommand)memsetCommand);
        subProgressMonitor.worked(2);
        if (result) {
          XmodelParameter xmodelPar = new XmodelParameter(file.length(), 0L);
          result = downloadFileByTrans(subProgressMonitor, result, xmodelPar, file, 95);
          if (!result)
            return result; 
        } 
      } 
      if (result && flashType == FlashType.spi) {
        result = sendCommand((SerialCommand)SFCommand.getInstance(chipID));
        subProgressMonitor.worked(1);
      } 
      TransferWay transferWay = getBurner().getBurnContext().getTransferWay();
      if (result) {
        subProgressMonitor.setTaskName("Erasing...");
        boolean isUsbAndSerialAndNonNude = (transferWay == TransferWay.UsbAndSerial || FastbootView.isSupportUsbNonNude(transferWay));
        if (isUsbAndSerialAndNonNude && FlashType.fuzzyCompareFlash(flashType, FlashType.emmc)) {
          infoln("\nSend command:\t " + eraseCommand.getCommand());
          result = CallLibUSBWorker.eraseFlashByUSB((int)address, (int)eraseLength, eraseCommand
              .getTimeout());
        } else {
          result = sendCommand((SerialCommand)eraseCommand);
        } 
      } 
      subProgressMonitor.worked(1);
      if (result && validFile) {
        if (transferWay == TransferWay.Uart2Compress) {
          UnZipCommand unZipCommand = UnZipCommand.unzip(getHexStartAddress(), unzipAddr);
          result = sendCommand(unZipCommand.toString(), unZipCommand
              .calculateTimeOut(Long.toHexString(tempFileDivisionSize)));
        } 
        if (this.burnTimeLogger != null && file != null)
          this.burnTimeLogger.logStart(file.getName(), writeLength); 
        subProgressMonitor.setTaskName("Writing...");
        result = sendCommand((SerialCommand)writeCommand);
        if (this.burnTimeLogger != null)
          this.burnTimeLogger.logEnd(); 
      } 
      if (result) {
        long realBurnLength = getRealBurnLength(eraseLength);
        if (realBurnLength > eraseLength) {
          returnError(BurntLengthGreaterEraseLength);
          return false;
        } 
      } 
      int burnCount = 1;
      while (!result) {
        String returnString = getCurReceived();
        if (returnString.contains("WARNING: block is marked as bad")) {
          warningln("A block is marked as bad block, burning the file again,the current is the " + burnCount + " times");
          if (this.burnTimeLogger != null && file != null)
            this.burnTimeLogger.logStart(file.getName(), writeLength); 
          subProgressMonitor.setTaskName("Writing...");
          result = sendCommand((SerialCommand)writeCommand);
          if (this.burnTimeLogger != null)
            this.burnTimeLogger.logEnd(); 
          burnCount++;
        } 
      } 
      return result;
    } finally {
      subProgressMonitor.done();
    } 
  }
  
  private void printLog(long burnStartTime) {
    long burnTotalTime = System.currentTimeMillis() - burnStartTime;
    String burnTotalTimeStr = calculateBurnTotalTime(burnTotalTime);
    getMonitor().getEvent().setTotalBurnTime(burnTotalTimeStr);
    TransferWay transferWay = getBurner().getBurnContext().getTransferWay();
    if (isNeedSetupNet(transferWay))
      getMonitor().println(this.tftpTimeLogger.exportLog("TFTP Transfer")); 
    this.burnTimeLogger.logEnd();
    getMonitor().println("\nTotal Burn Time: " + burnTotalTime + " ms");
  }
  
  public boolean upload(IProgressMonitor uiMonitor, List<PartitionItem> parts) throws IOException, ReceiveTimeOutException {
    setProgressMonitor(this.uimonitor);
    return false;
  }
  
  public boolean erase(IProgressMonitor uiMonitor, List<PartitionItem> parts) throws IOException, ReceiveTimeOutException {
    ToolStatistics.INSTANCE.logUsage("BurnTool", "Erase Partition", "ChipName: " + this.burner
        .getBurnContext().getDeviceSetting().getName());
    ToolStatistics.INSTANCE.logToolInUse("BurnTool");
    String versionInfo = getVersionInfo("Erase Partition");
    CallLibJTAGWorker.printDebugInfo(versionInfo);
    setProgressMonitor(uiMonitor);
    registerJnaFunction(uiMonitor, 100);
    int selected = 0;
    int totalWork = 0;
    for (PartitionItem partitionItem : parts) {
      if (partitionItem.isChecked())
        selected++; 
    } 
    totalWork = (selected + 2) * 100;
    uiMonitor.beginTask("", totalWork);
    boolean bRet = true;
    uiMonitor.setTaskName("Init Board");
    ArrayList<FlashType> hasFlashTypes = new ArrayList<>();
    for (PartitionItem partitionItem : parts) {
      if (partitionItem.isChecked()) {
        FlashType flashType = partitionItem.getFlashType();
        if (!hasFlashTypes.contains(flashType))
          hasFlashTypes.add(flashType); 
      } 
    } 
    if (getBurner().getBurnContext().getTransferWay() == TransferWay.USBNonNude) {
      bRet = bootromByNonNude(this.uimonitor);
      if (!bRet)
        return false; 
      if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
        loadBlockSizes(hasFlashTypes, ChipID.chip0, false);
      } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
        loadBlockSizes(hasFlashTypes, ChipID.chip0, false);
        warningln("Warning: Failed to load version info!");
      } else {
        returnError("Failed to load version info!");
        return false;
      } 
    } else {
      bRet = prepareToBurn(false, EventType.ERASE, hasFlashTypes);
      if (!bRet)
        return false; 
    } 
    uiMonitor.worked(100);
    uiMonitor.setTaskName("Erase Partitions");
    int partsSize = parts.size();
    for (int index = 0; index < partsSize; index++) {
      PartitionItem part = parts.get(index);
      if (part.isChecked()) {
        getMonitor().highlight(part);
        String partitionName = (part.getName() == null) ? "" : part.getName();
        uiMonitor.subTask("Partition: " + partitionName);
        long partLength = part.getLength();
        FlashType flashType = part.getFlashType();
        long partStart = part.getStart();
        long startSector = partStart / 512L;
        if (index == partsSize - 1 && partLength == -1L) {
          partLength = getLastPartitionLen(part, partStart, partLength, flashType, startSector);
          if (partLength == -1L)
            return false; 
        } 
        if (FlashType.spi == flashType)
          bRet = sendCommand((SerialCommand)SFCommand.getInstance(part.getChipID())); 
        if (bRet)
          bRet = sendCommand((SerialCommand)EraseCommand.erase(flashType, Long.toHexString(partStart), Long.toHexString(partLength))); 
        if (bRet) {
          infoln("Partition " + partitionName + " Erased successfully!");
        } else {
          returnError("Failed to Erase " + partitionName + " Partition!");
          sendResetCommand();
          return false;
        } 
        uiMonitor.worked(100);
      } 
    } 
    TransferWay transferWay = getBurner().getBurnContext().getTransferWay();
    boolean isResetJtag = ((FastbootView.isChipBranchSeries2() || FastbootView.isChipBranchSeries0()) && (transferWay == TransferWay.JTAG || transferWay == TransferWay.JTAGAndNet || transferWay == TransferWay.JTAGAndSerial));
    if (!isResetJtag) {
      String resetCommand = "reset";
      sendCommandNoReturnValue(resetCommand);
    } 
    return bRet;
  }
  
  public boolean eraseAll(IProgressMonitor uiMonitor, List<PartitionItem> children) throws IOException, ReceiveTimeOutException {
    ToolStatistics.INSTANCE.logUsage("BurnTool", "Erase All By Partition", "ChipName: " + this.burner
        .getBurnContext().getDeviceSetting().getName());
    ToolStatistics.INSTANCE.logToolInUse("BurnTool");
    String versionInfo = getVersionInfo("Erase All By Partition");
    CallLibJTAGWorker.printDebugInfo(versionInfo);
    setProgressMonitor(uiMonitor);
    registerJnaFunction(uiMonitor, 100);
    boolean bRet = true;
    int totalWork = 100;
    uiMonitor.setTaskName("Init Board");
    uiMonitor.beginTask("", totalWork);
    ArrayList<FlashType> hasFlashTypes = new ArrayList<>();
    hasFlashTypes.add(FlashType.spi);
    hasFlashTypes.add(FlashType.nand);
    if (getBurner().getBurnContext().getTransferWay() == TransferWay.USBNonNude) {
      bRet = bootromByNonNude(this.uimonitor);
      if (!bRet)
        return false; 
      if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
        loadBlockSizes(hasFlashTypes, ChipID.chip0, false);
      } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
        loadBlockSizes(hasFlashTypes, ChipID.chip0, false);
        warningln("Warning: Failed to load version info!");
      } else {
        returnError("Failed to load version info!");
        return false;
      } 
    } else {
      bRet = prepareToBurn(false, EventType.ERASE, hasFlashTypes);
      if (!bRet)
        return false; 
    } 
    uiMonitor.worked(10);
    uiMonitor.setTaskName("Erase Partitions");
    long nandSpaceSize = getNandSpaceSize();
    if (nandSpaceSize != 0L && nandSpaceSize > 0L) {
      EraseCommand eraseCommand = eraseAllCommand(nandSpaceSize, this.burner.getBurnContext().getDeviceSetting());
      bRet = sendCommand((SerialCommand)eraseCommand);
    } 
    long spiSpaceSize = getSpiSpaceSize();
    if (spiSpaceSize != 0L && spiSpaceSize > 0L) {
      bRet = sendCommand((SerialCommand)SFCommand.getInstance(ChipID.chip0));
      if (bRet)
        bRet = sendCommand((SerialCommand)EraseCommand.erase(FlashType.spi, Long.toHexString(0L), 
              Long.toHexString(spiSpaceSize))); 
      uiMonitor.worked(30);
    } 
    if (bRet) {
      infoln("Erase all partitions successfully!");
      TransferWay transferWay = getBurner().getBurnContext().getTransferWay();
      boolean isResetJtag = ((FastbootView.isChipBranchSeries2() || FastbootView.isChipBranchSeries0()) && (transferWay == TransferWay.JTAG || transferWay == TransferWay.JTAGAndNet || transferWay == TransferWay.JTAGAndSerial));
      if (!isResetJtag) {
        String resetCommand = "reset";
        sendCommandNoReturnValue(resetCommand);
      } 
    } else {
      returnError("Failed to Erase all partitions!");
      return false;
    } 
    uiMonitor.worked(100);
    return bRet;
  }
  
  public static EraseCommand eraseAllCommand(long nandSpaceSize, IDeviceSetting iDeviceSetting) {
    String eraseAllCommand = "nand erase 0";
    boolean isEraseChip = false;
    if (iDeviceSetting instanceof UnknownDevice)
      isEraseChip = ((UnknownDevice)iDeviceSetting).getIsEraseChip(); 
    if (isEraseChip)
      eraseAllCommand = "nand erase.chip"; 
    EraseCommand eraseCommand = new EraseCommand(eraseAllCommand);
    eraseCommand.setTimeout(EraseCommand.calTimeout(String.valueOf(nandSpaceSize), FlashType.nand));
    return eraseCommand;
  }
  
  protected long getLastPartitionLen(PartitionItem partitionItem, long partStart, long partLength, FlashType flashType, long startSector) throws IOException, ReceiveTimeOutException {
    long resultLength = partLength;
    if (FlashType.isEmmcOrUfs(flashType)) {
      long neMMCCapacity = geteMMCSpaceSize(flashType);
      if (0L == neMMCCapacity) {
        String errorInfo = "Failed to get eMMC capacity!";
        if (FlashType.fuzzyCompareFlash(flashType, FlashType.ufs))
          errorInfo = "Failed to get ufs capacity!"; 
        warningln(errorInfo);
        return -1L;
      } 
      resultLength = neMMCCapacity - startSector * 512L;
    } else if (flashType == FlashType.nand) {
      long nParCapacity = getNandParSpaceSize();
      if (nParCapacity == 0L) {
        warningln("Failed to get Parttition capacity!");
        return -1L;
      } 
      if (nParCapacity - partStart > 0L)
        resultLength = nParCapacity - partStart; 
    } else if (flashType == FlashType.spi) {
      boolean result = sendCommand((SerialCommand)SFCommand.getInstance(partitionItem.getChipID()));
      if (!result)
        return -1L; 
      long nParCapacity = getSpiParSpaceSize();
      if (nParCapacity == 0L) {
        warningln("Failed to get Parttition capacity!");
        return -1L;
      } 
      if (nParCapacity - partStart > 0L)
        resultLength = nParCapacity - partStart; 
    } 
    warningln("All Parttition remain space" + resultLength + " Bytes allotted to last partition!");
    partitionItem.setLength(resultLength);
    return resultLength;
  }
  
  public File getHome(List<PartitionItem> parts, EventType eventType) {
    if (parts.size() == 0)
      return null; 
    for (PartitionItem partitionItem : parts) {
      if (partitionItem.isChecked()) {
        File file = partitionItem.getFile();
        if (file != null) {
          if (eventType == EventType.BURN) {
            if (file.exists() && file.isFile())
              return file.getParentFile(); 
            continue;
          } 
          return file.getParentFile();
        } 
      } 
    } 
    for (PartitionItem partitionItem : parts) {
      File file = partitionItem.getFile();
      if (file != null)
        return file.getParentFile(); 
    } 
    return null;
  }
  
  private long getNandParSpaceSize() throws IOException, ReceiveTimeOutException {
    if (!sendCommand("getinfo nand"))
      return 0L; 
    String received = getCurReceived();
    return WorkerUtility.ReceiveHelper.getChipSize(received);
  }
  
  private long getSpiParSpaceSize() throws IOException, ReceiveTimeOutException {
    if (!sendCommand("getinfo spi"))
      return 0L; 
    String received = getCurReceived();
    return WorkerUtility.ReceiveHelper.getChipSize(received);
  }
  
  protected long getNandSpaceSize() throws IOException, ReceiveTimeOutException {
    if (!sendCommand("getinfo nand"))
      return 0L; 
    String received = getCurReceived();
    long result = WorkerUtility.ReceiveHelper.getChipSize(received);
    return result;
  }
  
  protected long getSpiSpaceSize() throws IOException, ReceiveTimeOutException {
    if (!sendCommand("getinfo spi"))
      return 0L; 
    String received = getCurReceived();
    long result = WorkerUtility.ReceiveHelper.getChipSize(received);
    if (FastbootView.isChipBranchSeries7())
      result = WorkerUtility.ReceiveHelper.getChipBranchSeries7ChipSize(received); 
    return result;
  }
  
  protected long geteMMCSpaceSize(FlashType flashType) throws IOException, ReceiveTimeOutException {
    String infocmd = "mmcinfo";
    if (flashType != null)
      if (FlashType.fuzzyCompareFlash(flashType, FlashType.ufs)) {
        infocmd = "ufsinfo";
      } else if (flashType == FlashType.ssd) {
        infocmd = "nvme info";
      }  
    if (!sendCommand(infocmd)) {
      warningln("Failed to send command " + infocmd + ".  ");
      return 0L;
    } 
    String received = getCurReceived();
    if (flashType != null && flashType == FlashType.ssd)
      return getCapacityBySsd(infocmd, received); 
    TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
    if (transferWay == TransferWay.UsbAndSerial || 
      FastbootView.isSupportUsbNonNude(transferWay))
      return getCapacityByUsbAndSerial(infocmd, received); 
    return getCapacityByEmmc(infocmd, received);
  }
  
  private long getCapacityByUsbAndSerial(String infocmd, String received) {
    String serialReceive = serialReceived.toString();
    long capacityValue = 0L;
    if (serialReceive.contains("Capacity: ")) {
      capacityValue = getCapacityByEmmc(infocmd, serialReceive);
      if (capacityValue > 0L)
        return capacityValue; 
    } 
    String key = "[EOT](OK)";
    int startIndex = received.indexOf(key);
    if (startIndex == -1) {
      warningln(infocmd + ":" + infocmd);
      warningln("[EOT](OK): " + startIndex);
      return 0L;
    } 
    startIndex += key.length();
    int endIndex = received.lastIndexOf(".");
    if (endIndex == -1) {
      warningln("." + startIndex);
      return 0L;
    } 
    String strCapacity = received.substring(startIndex, endIndex);
    try {
      capacityValue = Long.parseLong(strCapacity.trim());
    } catch (NumberFormatException numberFormatException) {
      warningln(Messages.EmmcComposite_Capacity_Error);
    } 
    return capacityValue;
  }
  
  private long getCapacityByEmmc(String infocmd, String received) {
    String key = "High Capacity: ";
    int startIndex = received.indexOf(key);
    if (startIndex == -1) {
      warningln(infocmd + ":" + infocmd);
      warningln("High Capacity: " + startIndex);
      return 0L;
    } 
    startIndex += key.length();
    key = "Capacity: ";
    startIndex = received.indexOf(key, startIndex);
    if (startIndex == -1) {
      warningln("Capacity: " + startIndex);
      return 0L;
    } 
    startIndex += key.length();
    int endIndex = received.indexOf("\r\n", startIndex);
    if (endIndex == -1) {
      warningln("\r\n" + endIndex);
      return 0L;
    } 
    String strCapacity = received.substring(startIndex, endIndex);
    long capacityValue = 0L;
    try {
      capacityValue = Long.parseLong(strCapacity.trim());
    } catch (NumberFormatException numberFormatException) {
      warningln(Messages.EmmcComposite_Capacity_Error);
    } 
    return capacityValue;
  }
  
  private long getCapacityBySsd(String infocmd, String received) {
    String key = "Capacity: ";
    int startIndex = received.indexOf(key);
    if (startIndex == -1) {
      warningln(infocmd + ":" + infocmd);
      warningln("Capacity: " + startIndex);
      return 0L;
    } 
    int endIndex = received.indexOf("MB", startIndex);
    if (endIndex == -1) {
      warningln("\r\n" + endIndex);
      return 0L;
    } 
    String strCapacity = received.substring(startIndex + key.length(), endIndex).trim();
    long capacityValue = 0L;
    try {
      capacityValue = (long)Double.parseDouble(strCapacity);
    } catch (NumberFormatException numberFormatException) {
      warningln(Messages.EmmcComposite_Capacity_Error);
    } 
    receivDebugInfo(key + key);
    return capacityValue * 1024L * 1024L;
  }
  
  private File prepareReplacedBootargs(File inputFile, String newString) {
    getMonitor().infoln("\nReplace 'mtdparts' definition in bootargs partition.");
    getMonitor().info("With new string: ");
    getMonitor().warnln(newString);
    String strLines = null;
    byte[] dstBytes = null;
    RandomAccessFile randFile = null;
    try {
      byte[] content = FileUtils.readAllBytes(inputFile.getCanonicalPath());
      if (content.length < 4)
        return inputFile; 
      byte[] strBytes = BytesUtils.genSubBytes(content, 4, content.length - 4);
      strLines = new String(strBytes, System.getProperty("file.encoding"));
      String[] lines = strLines.split("\000");
      String targetLine = null;
      int targetLineIndex = -1;
      for (int i = 0; i < lines.length; i++) {
        if (lines[i].startsWith("bootargs".toLowerCase())) {
          targetLine = lines[i];
          targetLineIndex = i;
          break;
        } 
      } 
      if (targetLine == null || targetLineIndex == -1)
        return inputFile; 
      int mtdpartsStart = targetLine.indexOf("mtdparts");
      if (mtdpartsStart == -1)
        return inputFile; 
      String other = "(others)";
      String sdCard = "(sdcard)";
      int otherLen = other.length();
      int sdCardLen = sdCard.length();
      String mtdpartsTitleString = targetLine.substring(mtdpartsStart);
      int partsLengthEmpty = mtdpartsTitleString.indexOf(" ");
      int partsLengthOther = mtdpartsTitleString.indexOf(other);
      int partsLengthSdCard = mtdpartsTitleString.indexOf(sdCard);
      int mtdpartsLength = -1;
      if (partsLengthOther != -1) {
        newString = newString.concat(",-(others)");
        mtdpartsLength = partsLengthOther + otherLen;
      } else if (partsLengthSdCard != -1) {
        mtdpartsLength = partsLengthSdCard + sdCardLen;
      } else if (partsLengthEmpty != -1) {
        mtdpartsLength = partsLengthEmpty;
      } else {
        mtdpartsLength = mtdpartsTitleString.length();
      } 
      if (mtdpartsLength == -1)
        return inputFile; 
      StringBuilder newTargetString = new StringBuilder();
      newTargetString.append(targetLine.substring(0, mtdpartsStart));
      newTargetString.append(newString);
      newTargetString.append(targetLine.substring(mtdpartsStart + mtdpartsLength));
      lines[targetLineIndex] = newTargetString.toString();
      StringBuilder strToWrite = new StringBuilder();
      for (int j = 0; j < lines.length; j++) {
        if (lines[j].length() == 0)
          break; 
        strToWrite.append(lines[j]);
        strToWrite.append("\000");
      } 
      dstBytes = strToWrite.toString().getBytes(System.getProperty("file.encoding"));
      int len = (int)bootargsFileLen;
      BytesUtils.copyBytes(content, dstBytes, 4, 0, dstBytes.length);
      ZlibCRC32 crc32 = new ZlibCRC32();
      crc32.update(content, 4, len - 4);
      int crcValue = (int)crc32.getValue();
      content[0] = (byte)(crcValue & 0xFF);
      content[1] = (byte)(crcValue >> 8);
      content[2] = (byte)(crcValue >> 16);
      content[3] = (byte)(crcValue >> 24);
      String parentFolder = inputFile.getParent();
      if (parentFolder == null)
        parentFolder = "/"; 
      String tempFileName = parentFolder + "/bootargs.temp";
      File tempFile = new File(tempFileName);
      if (tempFile.exists() && 
        !tempFile.delete())
        tempFile.deleteOnExit(); 
      randFile = new RandomAccessFile(tempFile, "rw");
      randFile.seek(0L);
      randFile.write(content);
      return new File(tempFileName);
    } catch (UnsupportedEncodingException e1) {
      return inputFile;
    } catch (IOException e) {
      return inputFile;
    } finally {
      try {
        if (randFile != null)
          randFile.close(); 
      } catch (IOException e) {
        return inputFile;
      } 
    } 
  }
  
  protected File prepareEmmcReplacedBootargs(File inputFile, String newString) {
    int emmcType = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getEmmcType();
    if (emmcType == EMMC_TYPE_E.EMMC_HAVE_MBR_TYPE.ordinal()) {
      getMonitor().infoln("\nReplace 'mtdparts' definition in bootargs partition.");
    } else if (emmcType == EMMC_TYPE_E.EMMC_NO_MBR_TYPE.ordinal()) {
      getMonitor().infoln("\nReplace 'blkdevparts' definition in bootargs partition.");
    } 
    getMonitor().info("With new string: ");
    getMonitor().warnln(newString);
    String strLines = null;
    byte[] dstBytes = null;
    RandomAccessFile randFile = null;
    try {
      byte[] content = FileUtils.readAllBytes(inputFile.getCanonicalPath());
      if (content.length < 4)
        return inputFile; 
      byte[] strBytes = BytesUtils.genSubBytes(content, 4, content.length - 4);
      strLines = new String(strBytes, System.getProperty("file.encoding"));
      String[] lines = strLines.split("\000");
      String targetLine = null;
      int targetLineIndex = -1;
      for (int i = 0; i < lines.length; i++) {
        if (lines[i].startsWith("bootargs".toLowerCase())) {
          targetLine = lines[i];
          targetLineIndex = i;
          break;
        } 
      } 
      if (targetLine == null || targetLineIndex == -1)
        return inputFile; 
      int partsStart = -1;
      if (emmcType == EMMC_TYPE_E.EMMC_HAVE_MBR_TYPE.ordinal()) {
        partsStart = targetLine.indexOf("mtdparts");
      } else if (emmcType == EMMC_TYPE_E.EMMC_NO_MBR_TYPE.ordinal()) {
        partsStart = targetLine.indexOf("blkdevparts");
      } 
      if (partsStart == -1)
        return inputFile; 
      String other = "(others)";
      String sdCard = "(sdcard)";
      int otherLen = other.length();
      int sdCardLen = sdCard.length();
      String partsTitleString = targetLine.substring(partsStart);
      int partsLengthEmpty = partsTitleString.indexOf(" ");
      int partsLengthOther = partsTitleString.indexOf(other);
      int partsLengthSdCard = partsTitleString.indexOf(sdCard);
      int partsLength = -1;
      if (partsLengthOther != -1) {
        newString = newString.concat(",-(others)");
        partsLength = partsLengthOther + otherLen;
      } else if (partsLengthEmpty != -1) {
        partsLength = partsLengthEmpty;
      } else if (partsLengthSdCard != -1) {
        partsLength = partsLengthSdCard + sdCardLen;
      } else {
        partsLength = partsTitleString.length();
      } 
      if (partsLength == -1)
        return inputFile; 
      StringBuilder newTargetString = new StringBuilder();
      newTargetString.append(targetLine.substring(0, partsStart));
      newTargetString.append(newString);
      newTargetString.append(targetLine.substring(partsStart + partsLength));
      lines[targetLineIndex] = newTargetString.toString();
      StringBuilder strToWrite = new StringBuilder();
      for (int j = 0; j < lines.length; j++) {
        if (lines[j].length() == 0);
        strToWrite.append(lines[j]);
        strToWrite.append("\000");
      } 
      dstBytes = strToWrite.toString().getBytes(System.getProperty("file.encoding"));
      int len = (int)bootargsFileLen;
      BytesUtils.copyBytes(content, dstBytes, 4, 0, dstBytes.length);
      ZlibCRC32 crc32 = new ZlibCRC32();
      crc32.update(content, 4, len - 4);
      int crcValue = (int)crc32.getValue();
      content[0] = (byte)(crcValue & 0xFF);
      content[1] = (byte)(crcValue >> 8);
      content[2] = (byte)(crcValue >> 16);
      content[3] = (byte)(crcValue >> 24);
      String parentFolder = inputFile.getParent();
      if (parentFolder == null)
        parentFolder = "/"; 
      String tempFileName = parentFolder + "/bootargs.temp";
      File tempFile = new File(tempFileName);
      if (tempFile.exists() && 
        !tempFile.delete())
        tempFile.deleteOnExit(); 
      randFile = new RandomAccessFile(tempFile, "rw");
      randFile.seek(0L);
      randFile.write(content);
      return new File(tempFileName);
    } catch (UnsupportedEncodingException e1) {
      return inputFile;
    } catch (IOException e) {
      return inputFile;
    } finally {
      try {
        if (randFile != null)
          randFile.close(); 
      } catch (IOException e) {
        return inputFile;
      } 
    } 
  }
  
  private String getBootargsPartitionString(List<PartitionItem> parts, String spiMtdId, String nandMtdId) {
    if ((spiMtdId == null || spiMtdId.length() == 0) && (nandMtdId == null || nandMtdId.length() == 0))
      return null; 
    StringBuilder sb = new StringBuilder();
    sb.append("mtdparts=");
    int spiCount = 0;
    int nandCount = 0;
    for (PartitionItem pi : parts) {
      if (pi.getFlashType() == FlashType.spi)
        spiCount++; 
      if (pi.getFlashType() == FlashType.nand)
        nandCount++; 
    } 
    if (spiMtdId != null && spiMtdId.length() > 0 && spiCount > 0) {
      sb.append(spiMtdId + ":");
      int writeCount = 0;
      for (PartitionItem part : parts) {
        if (part.getFlashType() == FlashType.nand)
          continue; 
        String partSize = "-";
        if (part.getLength() > -1L)
          partSize = StringUtils.getStorageSizeString(part.getLength()); 
        if (partSize == null)
          return null; 
        if (part.getName().equalsIgnoreCase("bootargs") && 
          part.getFile() != null)
          bootargsFileLen = part.getFile().length(); 
        if (writeCount > 0)
          sb.append(","); 
        sb.append(partSize + "(" + partSize + ")");
        writeCount++;
      } 
      if (nandMtdId != null && nandMtdId.length() > 0 && nandCount > 0)
        sb.append(";"); 
    } 
    if (nandMtdId != null && nandMtdId.length() > 0 && nandCount > 0) {
      sb.append(nandMtdId + ":");
      int writeCount = 0;
      for (PartitionItem part : parts) {
        if (part.getFlashType() == FlashType.spi)
          continue; 
        String partSize = "-";
        if (part.getLength() > -1L)
          partSize = StringUtils.getStorageSizeString(part.getLength()); 
        if (partSize == null)
          return null; 
        if (part.getName().equalsIgnoreCase("bootargs"))
          bootargsFileLen = part.getFile().length(); 
        if (writeCount > 0)
          sb.append(","); 
        sb.append(partSize + "(" + partSize + ")");
        writeCount++;
      } 
    } 
    return sb.toString();
  }
  
  public String getEmmcBootargsPartitionString(List<PartitionItem> parts) {
    int emmcType = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getEmmcType();
    StringBuilder sb = new StringBuilder();
    int writeCount = 0;
    if (emmcType == EMMC_TYPE_E.EMMC_HAVE_MBR_TYPE.ordinal()) {
      sb.append("mtdparts=soct_emmc:");
    } else if (emmcType == EMMC_TYPE_E.EMMC_NO_MBR_TYPE.ordinal()) {
      sb.append("blkdevparts=mmcblk0:");
    } 
    for (PartitionItem part : parts) {
      String partSize = "-";
      if (part.getLength() > -1L)
        partSize = StringUtils.getStorageSizeString(part.getLength()); 
      if (partSize == null)
        return null; 
      if (part.getName().equalsIgnoreCase("bootargs") && 
        part.getFile() != null)
        setBootargsFileLen(part.getFile().length()); 
      if (writeCount > 0)
        sb.append(","); 
      sb.append(partSize + "(" + partSize + ")");
      writeCount++;
    } 
    return sb.toString();
  }
  
  protected void checkDataAndWrite(String fillData, RandomAccessFile imagefile, int len) throws IOException {
    if (fillData.equals("0xFF")) {
      imagefile.write(BytesUtils.genBytesWithFF(len));
    } else if (fillData.equals("0x00")) {
      imagefile.write(BytesUtils.genBytesWithZero(len));
    } 
  }
  
  protected void fillDataToEmptyPart(String fillData, RandomAccessFile imagefile, long partitionLen) throws IOException {
    int emptyLenCount = (int)((partitionLen + 5242880L - 1L) / 5242880L);
    int emptyLenSpace = (int)(partitionLen % 5242880L);
    if (emptyLenCount == 0) {
      checkDataAndWrite(fillData, imagefile, (int)partitionLen);
    } else {
      for (int j = 0; j < emptyLenCount; j++) {
        if (j == emptyLenCount - 1) {
          if (emptyLenSpace == 0) {
            checkDataAndWrite(fillData, imagefile, 5242880);
          } else {
            checkDataAndWrite(fillData, imagefile, emptyLenSpace);
          } 
        } else {
          checkDataAndWrite(fillData, imagefile, 5242880);
        } 
      } 
    } 
  }
  
  protected void writeToFile(SubMonitor subProgressMonitor, RandomAccessFile imagefile, long offset, File partfile) throws IOException {
    try {
      String fillData = null;
      if (ToolUtils.isCmdLine()) {
        fillData = String.valueOf(PreferenceConfig.getImgFillData());
      } else {
        IPreferenceStore preferences = FastbootPreferences.getPreferenceStore();
        fillData = preferences.getString("vendor.burn.config.imageFill");
      } 
      imagefile.seek(offset);
      if (partfile.exists()) {
        FileInputStream in = null;
        byte[] buffer = new byte[5242880];
        if (fillData.equals("0xFF"))
          buffer = BytesUtils.genBytesWithFF(5242880); 
        try {
          in = new FileInputStream(partfile);
          long totalLength = partfile.length();
          subProgressMonitor.beginTask("Writing File\t" + partfile.getName(), 100);
          int read = -1;
          long written = 0L;
          int current = 0;
          while ((read = in.read(buffer)) > 0) {
            if (subProgressMonitor.isCanceled()) {
              getMonitor().getEvent().setUserCanceled(true);
              return;
            } 
            imagefile.write(buffer, 0, read);
            written += read;
            int temp = 0;
            if (totalLength != 0L)
              temp = (int)(written * 1.0D / totalLength * 100.0D); 
            if (temp > current) {
              subProgressMonitor.worked(temp - current);
              current = temp;
            } 
          } 
        } catch (FileNotFoundException fileNotFoundException) {
        
        } finally {
          if (in != null)
            try {
              in.close();
            } catch (IOException iOException) {} 
          buffer = null;
        } 
      } 
    } finally {
      subProgressMonitor.done();
    } 
  }
  
  protected String getFillData() {
    if (ToolUtils.isCmdLine())
      return String.valueOf(PreferenceConfig.getImgFillData()); 
    IPreferenceStore preferences = FastbootPreferences.getPreferenceStore();
    return preferences.getString("vendor.burn.config.imageFill");
  }
  
  public TimeLogger getZipAndPageTimeLogger() {
    return this.zipAndPageTimeLogger;
  }
  
  public TimeLogger getSpellingSparseFileLogger() {
    return this.spellingSparseFileLogger;
  }
  
  public String getTmpBootargsString() {
    return this.tmpBootargsString;
  }
  
  public void setTmpBootargsString(String tmpBootargsString) {
    this.tmpBootargsString = tmpBootargsString;
  }
  
  public TimeLogger getBurnBootTimeLogger() {
    return this.burnBootTimeLogger;
  }
  
  public void setBurnBootTimeLogger(TimeLogger burnBootTimeLogger) {
    this.burnBootTimeLogger = burnBootTimeLogger;
  }
  
  public TimeLogger getTftpTimeLogger() {
    return this.tftpTimeLogger;
  }
  
  public void setTftpTimeLogger(TimeLogger tftpTimeLogger) {
    this.tftpTimeLogger = tftpTimeLogger;
  }
  
  public TimeLogger getBurnTimeLogger() {
    return this.burnTimeLogger;
  }
  
  public void setBurnTimeLogger(TimeLogger burnTimeLogger) {
    this.burnTimeLogger = burnTimeLogger;
  }
  
  public TimeLogger getWaitZipTimeLogger() {
    return this.waitZipTimeLogger;
  }
  
  public void setWaitZipTimeLogger(TimeLogger waitZipTimeLogger) {
    this.waitZipTimeLogger = waitZipTimeLogger;
  }
  
  public TimeLogger getUnZipTimeLogger() {
    return this.unZipTimeLogger;
  }
  
  public void setUnZipTimeLogger(TimeLogger unZipTimeLogger) {
    this.unZipTimeLogger = unZipTimeLogger;
  }
  
  public TimeLogger getEraseTimeLogger() {
    return this.eraseTimeLogger;
  }
  
  public void setEraseTimeLogger(TimeLogger eraseTimeLogger) {
    this.eraseTimeLogger = eraseTimeLogger;
  }
  
  public TimeLogger getCrcLogger() {
    return this.crcLogger;
  }
  
  public void setCrcLogger(TimeLogger crcLogger) {
    this.crcLogger = crcLogger;
  }
  
  public void setZipAndPageTimeLogger(TimeLogger zipAndPageTimeLogger) {
    this.zipAndPageTimeLogger = zipAndPageTimeLogger;
  }
  
  public void setSpellingSparseFileLogger(TimeLogger spellingSparseFileLogger) {
    this.spellingSparseFileLogger = spellingSparseFileLogger;
  }
  
  public static long getBootargsFileLen() {
    return bootargsFileLen;
  }
  
  public static void setBootargsFileLen(long bootargsFileLen) {
    com.vendor.burn.serial.workers.PartitionBurnerWorker.bootargsFileLen = bootargsFileLen;
  }
}
